'use strict';

var view = require('@codemirror/view');
var language = require('@codemirror/language');
var highlight = require('@lezer/highlight');

// Forest theme color definitions
const base00 = '#1b2b34', // Background - dark blue-green forest shade
base01 = '#c0c5ce', // Foreground - soft cloudy light
base02 = '#343d46', // Selection and Selection Match - darker shade of forest mud
base03 = '#65737e', // Comment - greyish tinge resembling tree bark
base04 = '#c594c5', // Cursor - muted lavender, like early morning forest blooms
base05 = '#99c794', // Keyword, Tag - fresh leaf green
base06 = '#5fb3b3', // Variable, Parameter - reflective water surface tone
base07 = '#6699cc', // Function, Type - sky at dusk
base08 = '#ec5f67', // String, RegExp - autumn berry red
base09 = '#ab7967', // Constant, Number - earthy tone, wet autumn leaves
base0A = '#f99157', // Class, Heading - soft fox orange
invalid = '#f2777a'; // Invalid - faded red warning
const darkBackground = '#2b303b', // Dropdown background - deep shade found under thick leaves
highlightBackground = '#3e3d3257', // Active line and Matching bracket - moss on a rock
tooltipBackground = base01, // Tooltip using foreground color
cursor = base04, // Caret color
selection = '#398450c2', // Selection color
selectionMatch = '#5d3535';
// Define the editor theme styles for Forest.
const forestTheme = view.EditorView.theme({
    '&': {
        color: base01,
        backgroundColor: base00
    },
    '.cm-content': {
        caretColor: cursor
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor
    },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
        color: '#FFFFFF'
    },
    '.cm-panels': {
        backgroundColor: darkBackground,
        color: base01
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: '2px solid black'
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: '2px solid black'
    },
    '.cm-activeLine': {
        backgroundColor: highlightBackground
    },
    '.cm-gutters': {
        backgroundColor: darkBackground,
        color: base03
    },
    '.cm-activeLineGutter': {
        backgroundColor: selection,
        color: '#FFFFFF'
    },
    '.cm-tooltip': {
        backgroundColor: tooltipBackground
    },
    '.cm-selectionMatch': {
        backgroundColor: selectionMatch,
    },
    '.cm-searchMatch': {
        backgroundColor: base02, // Use the selection color for search matches
        color: base01, // Foreground color for text
        outline: `1px solid ${base03}` // Outline using comment color
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base05, // Use the keyword color for the selected search match
        color: base00 // Background color for high visibility
    },
    '.cm-searchMatch.cm-searchMatch-selected .ͼ5k': {
        color: base00
    },
    '.cm-searchMatch .cm-selectionMatch': {
        color: base01
    },
    '.cm-searchMatch .cm-selectionMatch .ͼ5k': {
        color: base01
    },
    '.cm-searchMatch .ͼ5k': {
        color: base01
    }
}, { dark: true });
// Define the highlighting style for code in the Forest theme.
const forestHighlightStyle = language.HighlightStyle.define([
    { tag: highlight.tags.keyword, color: base05 },
    { tag: [highlight.tags.name, highlight.tags.deleted, highlight.tags.character, highlight.tags.macroName], color: base06 },
    { tag: [highlight.tags.propertyName], color: base07 },
    { tag: [highlight.tags.processingInstruction, highlight.tags.string, highlight.tags.inserted, highlight.tags.special(highlight.tags.string)], color: base08 },
    { tag: [highlight.tags.function(highlight.tags.variableName), highlight.tags.labelName], color: base07 },
    { tag: [highlight.tags.color, highlight.tags.constant(highlight.tags.name), highlight.tags.standard(highlight.tags.name)], color: base09 },
    { tag: [highlight.tags.definition(highlight.tags.name), highlight.tags.separator], color: base06 },
    { tag: [highlight.tags.className], color: base0A },
    { tag: [highlight.tags.number, highlight.tags.changed, highlight.tags.annotation, highlight.tags.modifier, highlight.tags.self, highlight.tags.namespace], color: base09 },
    { tag: [highlight.tags.typeName], color: base07 },
    { tag: [highlight.tags.operator, highlight.tags.operatorKeyword], color: base05 },
    { tag: [highlight.tags.url, highlight.tags.escape, highlight.tags.regexp, highlight.tags.link], color: base08 },
    { tag: [highlight.tags.meta, highlight.tags.comment], color: base03 },
    { tag: highlight.tags.tagName, color: base05 },
    { tag: highlight.tags.strong, fontWeight: 'bold' },
    { tag: highlight.tags.emphasis, fontStyle: 'italic' },
    { tag: highlight.tags.link, textDecoration: 'underline' },
    { tag: highlight.tags.heading, fontWeight: 'bold', color: base0A },
    { tag: [highlight.tags.atom, highlight.tags.bool, highlight.tags.special(highlight.tags.variableName)], color: base06 },
    { tag: highlight.tags.invalid, color: invalid },
    { tag: highlight.tags.strikethrough, textDecoration: 'line-through' }
]);
// Extension to enable the Forest theme (both the editor theme and the highlight style).
const forest = [
    forestTheme,
    language.syntaxHighlighting(forestHighlightStyle)
];

exports.forest = forest;
exports.forestHighlightStyle = forestHighlightStyle;
exports.forestTheme = forestTheme;
