import { EditorView } from '@codemirror/view';
import { HighlightStyle, syntaxHighlighting } from '@codemirror/language';
import { tags } from '@lezer/highlight';

// Forest theme color definitions
const base00 = '#1b2b34', // Background - dark blue-green forest shade
base01 = '#c0c5ce', // Foreground - soft cloudy light
base02 = '#343d46', // Selection and Selection Match - darker shade of forest mud
base03 = '#65737e', // Comment - greyish tinge resembling tree bark
base04 = '#c594c5', // Cursor - muted lavender, like early morning forest blooms
base05 = '#99c794', // Keyword, Tag - fresh leaf green
base06 = '#5fb3b3', // Variable, Parameter - reflective water surface tone
base07 = '#6699cc', // Function, Type - sky at dusk
base08 = '#ec5f67', // String, RegExp - autumn berry red
base09 = '#ab7967', // Constant, Number - earthy tone, wet autumn leaves
base0A = '#f99157', // Class, Heading - soft fox orange
invalid = '#f2777a'; // Invalid - faded red warning
const darkBackground = '#2b303b', // Dropdown background - deep shade found under thick leaves
highlightBackground = '#3e3d3257', // Active line and Matching bracket - moss on a rock
tooltipBackground = base01, // Tooltip using foreground color
cursor = base04, // Caret color
selection = '#398450c2', // Selection color
selectionMatch = '#5d3535';
// Define the editor theme styles for Forest.
const forestTheme = /*@__PURE__*/EditorView.theme({
    '&': {
        color: base01,
        backgroundColor: base00
    },
    '.cm-content': {
        caretColor: cursor
    },
    '.cm-cursor, .cm-dropCursor': {
        borderLeftColor: cursor
    },
    '&.cm-focused > .cm-scroller > .cm-selectionLayer .cm-selectionBackground, .cm-selectionBackground, .cm-content ::selection': {
        backgroundColor: selection,
        color: '#FFFFFF'
    },
    '.cm-panels': {
        backgroundColor: darkBackground,
        color: base01
    },
    '.cm-panels.cm-panels-top': {
        borderBottom: '2px solid black'
    },
    '.cm-panels.cm-panels-bottom': {
        borderTop: '2px solid black'
    },
    '.cm-activeLine': {
        backgroundColor: highlightBackground
    },
    '.cm-gutters': {
        backgroundColor: darkBackground,
        color: base03
    },
    '.cm-activeLineGutter': {
        backgroundColor: selection,
        color: '#FFFFFF'
    },
    '.cm-tooltip': {
        backgroundColor: tooltipBackground
    },
    '.cm-selectionMatch': {
        backgroundColor: selectionMatch,
    },
    '.cm-searchMatch': {
        backgroundColor: base02, // Use the selection color for search matches
        color: base01, // Foreground color for text
        outline: `1px solid ${base03}` // Outline using comment color
    },
    '.cm-searchMatch.cm-searchMatch-selected': {
        backgroundColor: base05, // Use the keyword color for the selected search match
        color: base00 // Background color for high visibility
    },
    '.cm-searchMatch.cm-searchMatch-selected .ͼ5k': {
        color: base00
    },
    '.cm-searchMatch .cm-selectionMatch': {
        color: base01
    },
    '.cm-searchMatch .cm-selectionMatch .ͼ5k': {
        color: base01
    },
    '.cm-searchMatch .ͼ5k': {
        color: base01
    }
}, { dark: true });
// Define the highlighting style for code in the Forest theme.
const forestHighlightStyle = /*@__PURE__*/HighlightStyle.define([
    { tag: tags.keyword, color: base05 },
    { tag: [tags.name, tags.deleted, tags.character, tags.macroName], color: base06 },
    { tag: [tags.propertyName], color: base07 },
    { tag: [tags.processingInstruction, tags.string, tags.inserted, /*@__PURE__*/tags.special(tags.string)], color: base08 },
    { tag: [/*@__PURE__*/tags.function(tags.variableName), tags.labelName], color: base07 },
    { tag: [tags.color, /*@__PURE__*/tags.constant(tags.name), /*@__PURE__*/tags.standard(tags.name)], color: base09 },
    { tag: [/*@__PURE__*/tags.definition(tags.name), tags.separator], color: base06 },
    { tag: [tags.className], color: base0A },
    { tag: [tags.number, tags.changed, tags.annotation, tags.modifier, tags.self, tags.namespace], color: base09 },
    { tag: [tags.typeName], color: base07 },
    { tag: [tags.operator, tags.operatorKeyword], color: base05 },
    { tag: [tags.url, tags.escape, tags.regexp, tags.link], color: base08 },
    { tag: [tags.meta, tags.comment], color: base03 },
    { tag: tags.tagName, color: base05 },
    { tag: tags.strong, fontWeight: 'bold' },
    { tag: tags.emphasis, fontStyle: 'italic' },
    { tag: tags.link, textDecoration: 'underline' },
    { tag: tags.heading, fontWeight: 'bold', color: base0A },
    { tag: [tags.atom, tags.bool, /*@__PURE__*/tags.special(tags.variableName)], color: base06 },
    { tag: tags.invalid, color: invalid },
    { tag: tags.strikethrough, textDecoration: 'line-through' }
]);
// Extension to enable the Forest theme (both the editor theme and the highlight style).
const forest = [
    forestTheme,
    /*@__PURE__*/syntaxHighlighting(forestHighlightStyle)
];

export { forest, forestHighlightStyle, forestTheme };
